# This concern defines a percent complete value for the models.
#
# Each model must declare which attributes are concerned
# and what is the way to compute the percent.
# If no lambda is set the default one is !value.blank?.
#
# Exemple :
#
# percent :post, -> value { value != "foobar" }
# The attribute will not be considered as complete as
# long as its value is "foobar".
#
module Concerns::Percentable
  extend ActiveSupport::Concern

  included do
    # Compute the percent value before each save
    before_validation :set_percentage

    # Add a class attribute for excluded attributes
    class_attribute :excluded
    # Add a class attributes for the list of included attributes
    class_attribute :included

    # Add a field for the percent value.
    field :percent, type: Integer

    # Set default values.
    self.excluded = [:_id,:created_at,:updated_at,:_keywords,:percent]
    self.included = {}
  end

  module ClassMethods

    # Class method to define a percentable attribute
    # name : the name of the attribute
    # l : the lambda used to validate the value
    def percented(name, l=nil)
      included[name] = l
    end
  end

  private

  def set_percentage
    unless included.empty?
      p = self.class.fields.keys.inject(0) do |cumul, method|
        cumul + valid_cumulable_value(method.to_sym)
      end
      self.percent = (p * 100) / included.length
    end
  end

  def valid_cumulable_value(method)
    if included.include? method
      data = self.send method
      validator = included[method]
      valid = validator ? validator.call(data, self) : !data.blank?
    else
      valid = false
    end
    valid ? 1 : 0
  end
end

