# This concern defines a model as trackable.
# Creation, modifications and destruction will be saved as
# notifications in the corresponding users.
#
#
module Concerns::Trackable
  extend ActiveSupport::Concern

  included do
    before_save   :set_notifiables
    after_create  :notify_creation
    # after_update  :notify_modification
    after_destroy :notify_destruction

    # List of notifiable users for this model
    field :tracker_ids, type: Array, default: []
    field :modified_by, type: Moped::BSON::ObjectId
    field :embedded_in, type: Moped::BSON::ObjectId

    class_attribute :find_trackers
    class_attribute :find_trackable_as
    class_attribute :not_tracked

    self.find_trackers = {}
    self.find_trackable_as = {}
    self.not_tracked = {}
  end

  module ClassMethods
    # Pass a lambda finding the users that must be notified.
    def tracked_by(l)
      find_trackers[:method] = l
    end

    # Pass a lambda finding the name of the tracked model.
    def tracked_as(l)
      find_trackable_as[:method] = l
    end

    def not_tracked_if(l)
      not_tracked[:method] = l
    end
  end

  private

  def set_notifiables
    if !Rails.configuration.disable_trackers && self.tracker_ids.empty?
      admin_ids = UserQuery
        .new
        .all_admins
        .map {|admin| admin.id}
      self.tracker_ids = admin_ids.concat(find_trackers[:method].call(self))
    end
  end

  def notify_creation
    unless Rails.configuration.disable_trackers || (not_tracked[:method] && not_tracked[:method].call(:creation, self))
      tracked = find_trackable_as[:method] && find_trackable_as[:method].call(self)
      Sidekiq::Client.enqueue(TrackedNotificationsJob,
                              :creation,
                              self.tracker_ids,
                              self.id,
                              tracked && tracked[:name],
                              tracked && tracked[:klass],
                              self.embedded_in,
                              self.modified_by,
                              nil)
    end
  end

  def notify_modification
    unless Rails.configuration.disable_trackers || (not_tracked[:method] && not_tracked[:method].call(:modification, self))
      tracked = find_trackable_as[:method] && find_trackable_as[:method].call(self)
      Sidekiq::Client.enqueue(TrackedNotificationsJob,
                              :modification,
                              self.tracker_ids,
                              self.id,
                              tracked && tracked[:name],
                              tracked && tracked[:klass],
                              self.embedded_in,
                              self.modified_by,
                              self.previous_changes)
    end
  end

  def notify_destruction
    unless Rails.configuration.disable_trackers || (not_tracked[:method] && not_tracked[:method].call(:destruction, self))
      tracked = find_trackable_as[:method] && find_trackable_as[:method].call(self)
      Sidekiq::Client.enqueue(TrackedNotificationsJob,
                              :destruction,
                              self.tracker_ids,
                              nil,
                              tracked && tracked[:name],
                              tracked && tracked[:klass],
                              self.embedded_in,
                              self.modified_by,
                              nil)
    end
  end
end

