class Vintage
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::Search
  include Concerns::Percentable
  include Concerns::Trackable
  include ActiveModel::ForbiddenAttributesProtection

  after_save :set_computed_wine_percent

  mount_uploader :wine_label, WineLabelUploader

  field :title, type: String
  field :production_year, type: Integer
  field :alcohol_degree, type: Float
  field :serving_temperature, type: String
  field :storage_duration, type: String
  field :decantaring_duration, type: String
  field :price, type: Float
  field :grapes, type: Array, default: []
  field :awards, type: Array, default: []
  field :notes, type: Array, default: []
  field :tasting_comment, type: String
  field :tasting_opportunities, type: Array, default: []
  field :tasting_recipes, type: Array, default: []
  field :tasting_flavors, type: Array, default: []
  field :tasting_food_entries, type: Array, default: []
  field :tasting_food_dishes, type: Array, default: []
  field :tasting_food_desserts, type: Array, default: []
  field :tasting_food_cheeses, type: Array, default: []
  field :is_base, type: Boolean
  field :barcode, type: String
  field :reco_id, type: String
  field :tasting_profile, type: String
  field :average_rank, type: Float, default: 0
  field :unofficial, type: Boolean, default: false

  percented :production_year, -> value, entity {
    (entity.wine.interpro_id.to_s == "5229bd1a4d003d5e28000004") || # do not use for Armagnac
    !(value.blank?)
  }
  percented :alcohol_degree
  percented :serving_temperature
  percented :storage_duration, -> value, entity {
    (entity.wine.interpro_id.to_s == "5229bd1a4d003d5e28000004") || # do not use for Armagnac
    !(value.blank?)
  }
  percented :decantaring_duration
  percented :price
  percented :grapes, -> value, entity {
    !(
      (value && value.size.zero?) ||
      (value && value.size == 1 &&
       value[0]["key"] &&
       value[0]["value"].blank?)
    )
  }
  percented :tasting_comment
  percented :tasting_opportunities
  percented :tasting_recipes
  percented :tasting_flavors
  percented :tasting_food_entries
  percented :tasting_food_dishes
  percented :tasting_food_desserts
  percented :tasting_food_cheeses
  percented :wine_label

  tracked_by -> (model) { [model.wine.producer_id, model.wine.interpro_id] }
  tracked_as -> (model) { {name: model.title, klass: "Vintage" } }
  not_tracked_if -> (method, model) {
    (method == :creation || method == :destruction) && model.is_base
  }

  search_in :production_year, :tasting_recipes, :tasting_opportunities,
    :tasting_flavors, :tasting_food_entries, :tasting_food_dishes,
    :tasting_food_desserts, :tasting_food_cheeses, :storage_duration,
    :tasting_opportunities

  embedded_in :wine, inverse_of: :vintages

  embeds_one :award, cascade_callbacks: true

  has_many :user_comments

  default_scope -> { order_by(title: :desc) }

  validates_presence_of :title


  def displayed_name
    "#{self.wine.name} - #{self.title}"
  end


  def update_average_rank!
    if self.user_comments.size > 0
      self.average_rank = self.user_comments.inject(0.0) { |sum, el| sum + el.rank } / self.user_comments.size
    else
      self.average_rank = 0
    end
    self.save!
  end


  def official?
    !self.unofficial
  end


  def unofficial?
    self.unofficial
  end


  private

  def set_computed_wine_percent
    self.wine.save
  end
end
