require "mongoid_search"

class Wine
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::Search
  include Concerns::Percentable
  include Concerns::Trackable
  include ActiveModel::ForbiddenAttributesProtection

  default_scope -> { order_by(created_at: :asc) }

  after_validation :set_computed_wine_percent

  field :name, type: String
  field :add_name, type: String
  field :color, type: Moped::BSON::ObjectId
  field :sparkling_type, type: String
  field :appelation, type: String
  field :vinification, type: String
  field :culture_types, type: Array, default: []
  field :production_area, type: String
  field :country, type: Hash
  field :grounds, type: Array, default: []
  field :harvests, type: Array, default: []
  field :climates, type: Array, default: []
  field :allergens, type: Array, default: []
  field :ageing, type: Array, default: []
  field :status, type: Symbol, default: :draft
  field :interpro_id, type: Moped::BSON::ObjectId
  field :computed_percent, type: Integer
  field :unofficial, type: Boolean, default: false

  index interpro_id: 1
  index computed_percent: 1
  index updated_at: 1
  index created_at: 1
  index "vintages.title" => 1

  percented :add_name
  percented :color
  percented :appelation
  percented :vinification
  percented :culture_types
  percented :production_area
  percented :country
  percented :grounds
  percented :harvests
  percented :climates
  percented :allergens
  percented :ageing, -> value, entity {
    !( (value && value.size.zero?) ||
      (value && value.size == 1 && value[0]["key"] && value[0]["value"].blank?)
     )
  }

  tracked_by -> (model) { [model.producer_id, model.interpro_id] }
  tracked_as -> (model) { {name: model.name, klass: "Wine" } }

  belongs_to :producer, class_name: "User", inverse_of: :wines, index: true
  belongs_to :estate, index: true
  embeds_many :vintages, inverse_of: :wine

  has_many :user_comments

  validates_presence_of :name, :interpro_id

  search_in :name,
    :vintages => [
      :tasting_opportunities,
      :tasting_flavors,
      :tasting_food_entries,
      :tasting_food_dishes,
      :tasting_food_desserts,
      :tasting_food_cheeses]


  def published?
    self.status == :published
  end


  def draft?
    self.status != :published
  end

  def official?
    !self.unofficial
  end


  def unofficial?
    self.unofficial
  end


  def publish!
    # save
    # if computed_percent >= 50
    #   self.status = :published
    #   save
    # end
    self.set(status: :published)
  end


  def all_vintages_labeled?
    c = self.vintages.inject(0) do |count, v|
      (v.wine_label && v.wine_label.url) ? count + 1 : count
    end
    c == self.vintages.size
  end

  private

  def set_computed_wine_percent
    wine_percent = (((self.percent || 1) + vintages_percent) / 2)
    self.computed_percent = wine_percent
  end

  def vintages_percent
    nb_vintages = self.vintages.count { |v| v.persisted? }
    p = self.vintages.inject(0) do |tot, v|
      tot + ( v.persisted? ? v.percent : 0)
    end
    p / (nb_vintages > 0 ? nb_vintages : 1)
  end
end
