module WinewooCore::Repositories::Mongo
  class CriterionsMongoRepo

    def find(filters)
      @filters = filters
      category = filters.category && filters.category.parameterize.underscore.to_sym
      return category && find_and_serialize_criterions(category)
    end


    private

    def find_and_serialize_criterions(category)
      finder = FinderBuilder.build(category, @filters)
      found_criterions = finder.find
      finder.map(found_criterions)
    end



    class FinderBuilder

      def self.build(category, filters)
        case category
        when :appelations
          AppelationsFinder.new(filters)
        when :aoc
          AocsFinder.new(filters)
        when :grapes
          GrapesFinder.new(filters)
        when :storages
          StoragesFinder.new(filters)
        when :opportunities
          OpportunitiesFinder.new(filters)
        else
          PictosFinder.new(category, filters)
        end
      end
    end



    class Finder

      def initialize(filters)
        @filters = filters
      end


      def map(found_criterions)
        found_criterions.map do |criterion|
          serialize_criterion(criterion) unless criterion.blank?
        end.compact
      end


      private

      def serialize_criterion(criterion)
        criterion.is_a?(String) ?
          { picto: nil, name: criterion, value: criterion } :
          { picto: criterion.read_attribute(@filters.screen_size || :iphonex2_picto),
            name: criterion.name,
            value: criterion.id }
      end
    end



    class AppelationsFinder < Finder

      def find
        WinewooCore::Repositories::Mongo::Queries::InterproQuery.new.all.map do |interpro|
          if interpro.wine_data_element &&
              interpro.wine_data_element.appelations.any? &&
              any_wines?(interpro)
            {
              interpro: interpro.name,
              interpro_id: interpro.id,
              appelations: interpro.wine_data_element.appelations
            }
          end
        end
      end


      def map(found_criterions)
        return found_criterions.compact
      end


      private

      def any_wines?(interpro)
        WinewooCore::Repositories::Mongo::Queries::WineQuery.new.published.from_interpro(interpro.id).count > 0
      end
    end



    class AocsFinder < Finder

      def find
        interpro = @filters.interpro
        return [] unless interpro && interpro.wine_data_element
        interpro.wine_data_element.appelations
      end
    end



    class GrapesFinder < Finder

      def find
        res = GlobalWineData.all.first.grapes
        return @filters.query ?
          res.select{ |grape| grape.match(/^(#{@filters.query})/i) } :
          res
      end
    end



    class StoragesFinder < Finder

      def find
        GlobalWineData.all.first.storage_durations
      end
    end



    class OpportunitiesFinder < Finder

      def find
        GlobalWineData.all.first.tasting_opportunities
      end
    end



    class PictosFinder < Finder

      def initialize(category, query_param)
        super(query_param)
        @category = category
      end


      def find
        res = WinewooCore::Repositories::Mongo::Queries::PictoQuery.new.from_category(@category).by_name
        res = res.full_text_search @filters.query if @filters.query
        return res.page(@filters.page || 1)
          .per(@filters.per_page || Kaminari::config.default_per_page)
      end
    end
  end
end
