module WinewooCore::Repositories::Mongo
  class InterproCriterionsMongoRepo

    def find(filters)
      @filters = filters
      category = filters.category && filters.category.parameterize.underscore.to_sym
      return category && find_and_serialize_criterions(category)
    end


    private

    def find_and_serialize_criterions(category)
      found_criterions = FinderBuilder.build(category, @filters).find
      found_criterions.map do |criterion|
        serialize_criterion(criterion) unless criterion.blank?
      end.compact
    end


    def serialize_criterion(criterion)
      criterion.is_a?(String) ?
        { picto: nil, name: criterion, value: criterion } :
        { picto: criterion.read_attribute(@filters.screen_size || :iphonex2_picto),
          name: criterion.name,
          value: criterion.id }
    end



    class FinderBuilder

      def self.build(category, filters)
        case category
        when :appelations
          AocsFinder.new(filters)
        when :aoc
          AocsFinder.new(filters)
        when :grapes
          GrapesFinder.new(filters)
        when :storages
          StoragesFinder.new(filters)
        when :opportunities
          OpportunitiesFinder.new(filters)
        else
          PictosFinder.new(category, filters)
        end
      end
    end



    class Finder

      def initialize(filters)
        @filters = filters
      end
    end



    class AocsFinder < Finder

      def find
        interpro = @filters.interpro
        return [] unless interpro && interpro.wine_data_element
        interpro.wine_data_element.appelations
      end
    end



    class GrapesFinder < Finder

      def find
        res = GlobalWineData.all.first.grapes
        res.select{ |grape| grape.match(/#{@filters.query}/) } if @filters.query
        return res
      end
    end



    class StoragesFinder < Finder

      def find
        GlobalWineData.all.first.storage_durations
      end
    end



    class OpportunitiesFinder < Finder

      def find
        GlobalWineData.all.first.tasting_opportunities
      end
    end



    class PictosFinder < Finder

      def initialize(category, query_param)
        super(query_param)
        @category = category
      end


      def find
        if @category == :colors
          colors = @filters.interpro.wine_data_element.wine_colors
          colors.map {|id| PictoElement.find(id) }
        else
          res = WinewooCore::Repositories::Mongo::Queries::PictoQuery.new.from_category(@category).by_name
          res = res.full_text_search @filters.query if @filters.query
          return res.page(@filters.page || 1)
            .per(@filters.per_page || Kaminari::config.default_per_page)
        end
      end
    end
  end
end
